package app.memo;

import api.cms.Item;
import api.util.ISO8601;
import api.util.JSON;
import groovy.json.JsonOutput;
import jp.co.mintjams.osgi.service.jcr.script.ScriptingContext;

class Memo extends Item {
    Memo(context) {
        super(context);
    }

	static def create(ScriptingContext context) {
		return new Memo(context);
	}

	def getMetadata() {
		def e = [
			"identifier": getString("memo.identifier"),
			"text": content,
			"color": getString("memo.color"),
			"tag": getStringArray("memo.tag"),
			"image": [],
			"created": ISO8601.formatDate(created),
			"lastModified": ISO8601.formatDate(lastModified),
			"remainder": [
			    "enabled": false
			]
		];
		def imagesFolder = parent.getItem(getString("memo.identifier"));
		if (imagesFolder.exists() && imagesFolder.isCollection()) {
			for (imageFile in imagesFolder.list()) {
				if (imageFile.isCollection()) {
					continue;
				}
				e.image.add(imageFile.toObject());
			}
		}
		if (contains("memo.remainder.enabled")) {
			e.remainder.enabled = getBoolean("memo.remainder.enabled");
			if (contains("memo.remainder.date")) {
				e.remainder["date"] = ISO8601.formatDate(getDate("memo.remainder.date"));
			}
		}
		return e;
	}

    def getMetadataAsJson() {
        return JsonOutput.toJson(getMetadata());
    }
}
