package app.task;

import api.cms.Item;
import api.cms.ItemHelper;
import api.cms.MultipartUpload;
import api.security.Authorizable;
import api.util.ISO8601;
import api.util.JSON;
import groovy.json.JsonOutput;
import jp.co.mintjams.osgi.service.jcr.script.ScriptingContext;

class Task extends Item {
    def appConfig;
    def authorizables = [:];

    Task(context) {
        super(context);
    }

	static def create(ScriptingContext context, appConfig) {
	    def instance = new Task(context);
	    instance.appConfig = appConfig;
		return instance;
	}

	def findByIdentifier(identifier) {
		def path = "/var/webtop/apps/" + appConfig.identifier + "/tasks/" + identifier.substring(0, 2) + "/" + identifier.substring(2, 4) + "/" + identifier + "/task.txt";
		def r = context.resourceResolver.getResource(path);
		if (!r.exists()) {
		    return this;
		}
		return with(r);
	}

	def createNewFile() {
		def basePath = "/var/webtop/apps/" + appConfig.identifier + "/tasks";
		def baseItem = Item.create(context).findByPath(basePath);
		baseItem.mkdirs();
		for (def i = 0; i < 3; i++) {
			def identifier = java.util.UUID.randomUUID().toString();
			def item = baseItem.getItem(identifier.substring(0, 2) + "/" + identifier.substring(2, 4) + "/" + identifier + "/task.txt");
			if (!item.exists()) {
        		item.parent.mkdirs();
        		item.createNewFile();
        		item.setAttribute("task.identifier", identifier);
        		item.setAttribute("task.isTopic", true);
        		return with(item.resource);
			}
		}

		throw new java.lang.IllegalStateException("Could not create a new task.");
	}

	def createNewPost() {
        if (!exists()) {
    		throw new java.lang.IllegalStateException("Task does not exists.");
        }

        def postList = posts.reverse();
        postList.add(this);
        def last = postList.get(0);

        def baseItem = getItem("posts");
		baseItem.mkdirs();
		for (def i = 0; i < 3; i++) {
			def identifier = java.util.UUID.randomUUID().toString();
			def item = baseItem.getItem(identifier + "/task.txt");
			if (!item.exists()) {
        		item.parent.mkdirs();
        		item.createNewFile();
        		item.setAttribute("task.identifier", identifier);
        		item.setAttribute("task.isTopic", false);
        		item.setAttribute("task.assignee", context.repositorySession.userID);
        		item.setAttribute("task.isDraft", true);
        		item.setAttribute("task.subject", last.subject);
        		item.setAttribute("task.color", last.color);
        		item.setAttribute("task.tags", last.tags);
        		item.setAttribute("task.dueDate", last.dueDate);
        		item.setAttribute("task.followUpDate", last.followUpDate);
        		item.setAttribute("task.status", last.status);
        		item.setAttribute("task.candidateUsers", last.candidateUsers);
        		item.setAttribute("task.candidateGroups", last.candidateGroups);
        		return Task.create(context, appConfig).with(item.resource);
			}
		}

		throw new java.lang.IllegalStateException("Could not create a new post.");
	}

	def getOrCreatePost() {
        if (!exists()) {
    		throw new java.lang.IllegalStateException("Task does not exists.");
        }

        def postList = posts;
        if (!postList.isEmpty()) {
            def last = postList.get(postList.size() - 1);
            if (last.isDraft()) {
                if (last.assignee == context.repositorySession.userID) {
                    return last;
                }
                last.remove();
            }
        }

        return createNewPost();
	}

	def getIdentifier() {
        if (!exists()) {
            return null;
        }
        return getString("task.identifier");
	}

	def getSubject() {
        if (!exists()) {
            return null;
        }
        return getString("task.subject");
	}
    def setSubject(subject = "") {
        return setAttribute("task.subject", subject);
    }

	def getComments() {
        if (!exists()) {
            return null;
        }
        return getContent();
	}
    def setComments(comments = "") {
        if (!comments) {
            comments = "";
        }
        return setContent(comments);
    }

	def getColor() {
        if (!exists()) {
            return null;
        }
        return getString("task.color");
	}
    def setColor(color = "") {
        return setAttribute("task.color", color as String);
    }

	def getTags() {
        if (!exists()) {
            return null;
        }
        return getStringArray("task.tags");
	}
    def setTags(tags) {
        if (!tags) {
            return removeAttribute("task.tags");
        }
        if (tags instanceof String) {
            tags = [tags];
        }
        return setAttribute("task.tags", tags as String[]);
    }

	def getDueDate() {
        if (!exists()) {
            return null;
        }
        if (!contains("task.dueDate")) {
            return null;
        }
        return getDate("task.dueDate");
	}
    def setDueDate(dueDate) {
        if (!dueDate) {
            return removeAttribute("task.dueDate");
        }
        if (dueDate instanceof String) {
            dueDate = ISO8601.parseDate(dueDate);
        }
        return setAttribute("task.dueDate", dueDate as Date);
    }

	def getFollowUpDate() {
        if (!exists()) {
            return null;
        }
        if (!contains("task.followUpDate")) {
            return null;
        }
        return getDate("task.followUpDate");
	}
    def setFollowUpDate(followUpDate) {
        if (!followUpDate) {
            return removeAttribute("task.followUpDate");
        }
        if (followUpDate instanceof String) {
            followUpDate = ISO8601.parseDate(followUpDate);
        }
        return setAttribute("task.followUpDate", followUpDate as Date);
    }

	def getStatus() {
        if (!exists()) {
            return null;
        }
        return getString("task.status");
	}
    def setStatus(status = "") {
        return setAttribute("task.status", status as String);
    }

	def getAssignee() {
        if (!exists()) {
            return null;
        }
        return getString("task.assignee");
	}
    def setAssignee(assignee) {
        if (!assignee) {
            return removeAttribute("task.assignee");
        }
        return setAttribute("task.assignee", assignee as String);
    }

	def getCandidateUsers() {
        if (!exists()) {
            return null;
        }
        return getStringArray("task.candidateUsers");
	}
    def setCandidateUsers(candidateUsers) {
        if (!candidateUsers) {
            return removeAttribute("task.candidateUsers");
        }
        if (candidateUsers instanceof String) {
            candidateUsers = [candidateUsers];
        }
        return setAttribute("task.candidateUsers", candidateUsers as String[]);
    }

	def getCandidateGroups() {
        if (!exists()) {
            return null;
        }
        return getStringArray("task.candidateGroups");
	}
    def setCandidateGroups(candidateGroups) {
        if (!candidateGroups) {
            return removeAttribute("task.candidateGroups");
        }
        if (candidateGroups instanceof String) {
            candidateGroups = [candidateGroups];
        }
        return setAttribute("task.candidateGroups", candidateGroups as String[]);
    }

	def isTopic() {
        if (!exists()) {
            return false;
        }
        return getBoolean("task.isTopic");
	}

	def isDraft() {
        if (!exists()) {
            return false;
        }
        return getBoolean("task.isDraft");
	}
    def setDraft(isDraft = false) {
        return setAttribute("task.isDraft", isDraft as boolean);
    }

    def setAttachments(attachments) {
		def attachmentsFolder = parent.getItem("attachments");
		attachmentsFolder.mkdirs();
		for (u in attachments) {
			if (u.removed) {
				def attachmentFile = attachmentsFolder.getItem(u.filename);
				if (attachmentFile.exists()) {
					attachmentFile.remove();
				}
				continue;
			}

			if (u.uploadID) {
				def mu = MultipartUpload.create(context).resolve(u.uploadID);
				if (!mu.exists()) {
                	throw new java.lang.IllegalArgumentException("Could not obtain a uploaded file: " + u.uploadID);
				}

				def attachmentFile = attachmentsFolder.getItem(u.filename);
				if (!attachmentFile.exists()) {
					attachmentFile.createNewFile();
				}
				ItemHelper.create(context).with(attachmentFile).importContent(mu, u.mimeType);
				attachmentFile.setAttribute("jcr:lastModified", new Date());
				continue;
			}

			if (u.path) {
				def srcItem = Item.create(context).findByPath(u.path);
				if (!srcItem.exists()) {
                	throw new java.lang.IllegalArgumentException("Could not obtain a file: " + u.path);
				}

				def attachmentFile = attachmentsFolder.getItem(srcItem.name);
				if (attachmentFile.path == srcItem.path) {
                	throw new java.lang.IllegalArgumentException("The file is already uploaded: " + u.path);
				}
				if (!attachmentFile.exists()) {
					attachmentFile.createNewFile();
				}
				srcItem.copyTo(attachmentFile.path);
				attachmentFile.setAttribute("jcr:lastModified", new Date());
				continue;
			}
		}
        return this;
    }

	def getPosts() {
        if (!exists()) {
            return [];
        }

        def posts = [];
		def postsFolder = parent.getItem("posts");
		if (postsFolder.exists()) {
    		for (f in postsFolder.list()) {
    		    if (!f.isCollection()) {
    		        continue;
    		    }
    		    def p = f.getItem("task.txt");
    		    if (!p.exists()) {
    		        continue;
    		    }
    		    def task = Task.create(context, appConfig).with(p.resource);
    		    task.authorizables = authorizables;
    		    posts.add(task);
    		}
		}
		posts.sort { o1, o2 -> o1.created <=> o2.created };
		return posts;
	}

    def canAssignToMe() {
        if (!exists()) {
            return false;
        }

        if (!isTopic()) {
            return false;
        }

        def postList = posts.reverse();
        postList.add(this);
        def last = postList.get(0);

        if (last.isDraft()) {
            return false;
        }
        if (last.assignee) {
            return false;
        }
        if (last.candidateUsers.contains(context.repositorySession.userID)) {
            return true;
        }
        def user = Authorizable.create(context).findByName(context.repositorySession.userID).toObject();
        for (id in user.groups) {
			if (last.candidateGroups.contains(id)) {
				return true;
			}
        }
		return false;
    }

    def assignToMe() {
        if (!canAssignToMe()) {
    		throw new java.lang.IllegalStateException("Could not change the assignee.");
        }
        
        def postList = posts.reverse();
        def last;
        for (p in postList) {
            if (p.isDraft()) {
                p.remove();
                continue;
            }
            last = p;
            break;
        }
        if (!last) {
            last = this;
        }

	    def newPost = createNewPost();
		newPost.setSubject(last.subject);
		newPost.setComments("");
		newPost.setColor(last.color);
		newPost.setTags(last.tags);
		newPost.setDueDate(last.dueDate);
		newPost.setFollowUpDate(last.followUpDate);
		newPost.setAssignee(context.repositorySession.userID);
		newPost.setCandidateUsers(last.candidateUsers);
		newPost.setCandidateGroups(last.candidateGroups);
		newPost.setDraft(false);
		newPost.setAttribute("jcr:lastModified", new Date());
		return newPost;
    }

    def canPost() {
        if (!exists()) {
            return false;
        }

        if (!isTopic()) {
            return false;
        }

        def postList = posts.reverse();
        postList.add(this);
        def last = postList.get(0);

        if (last.assignee == context.repositorySession.userID) {
            return true;
        }
		return false;
    }

    def remove() {
        if (!exists()) {
    		throw new java.lang.IllegalStateException("Task does not exists.");
        }

        parent.remove();
    }

	def getMetadata() {
		def e = [
			"identifier": identifier,
			"subject": subject,
			"comments": comments,
			"color": color,
			"tag": tags,
			"dueDate": dueDate ? ISO8601.formatDate(dueDate) : null,
			"followUpDate": followUpDate ? ISO8601.formatDate(followUpDate) : null,
			"status": status,
			"assignee": assignee,
			"candidateUser": candidateUsers,
			"candidateGroup": candidateGroups,
			"created": ISO8601.formatDate(created),
			"createdBy": createdBy,
			"lastModified": ISO8601.formatDate(lastModified),
			"lastModifiedBy": lastModifiedBy,
			"attachments": [],
			"posts": [],
			"isDraft": isDraft()
		];

		def attachmentsFolder = parent.getItem("attachments");
		if (attachmentsFolder.exists()) {
    		for (f in attachmentsFolder.list()) {
    		    if (f.isCollection()) {
    		        continue;
    		    }
    		    e.attachments.add(f.toObject());
    		}
		}

        posts.each { p ->
            if (p.isDraft()) {
                if (p.assignee != context.repositorySession.userID) {
                    return;
                }
            }
		    e.posts.add(p.metadata);
        }

        if (!authorizables[createdBy]) {
            def a = Authorizable.create(context).findByName(createdBy);
            if (a.exists()) {
                authorizables[createdBy] = a.toObject();
            }
        }
        if (!authorizables[lastModifiedBy]) {
            def a = Authorizable.create(context).findByName(lastModifiedBy);
            if (a.exists()) {
                authorizables[lastModifiedBy] = a.toObject();
            }
        }
        if (assignee && !authorizables[assignee]) {
            def a = Authorizable.create(context).findByName(assignee);
            if (a.exists()) {
                authorizables[assignee] = a.toObject();
            }
        }
        candidateUsers.each { id ->
            if (!authorizables[id]) {
                def a = Authorizable.create(context).findByName(id);
                if (a.exists()) {
                    authorizables[id] = a.toObject();
                }
            }
        }
        candidateGroups.each { id ->
            if (!authorizables[id]) {
                def a = Authorizable.create(context).findByName(id);
                if (a.exists()) {
                    authorizables[id] = a.toObject();
                }
            }
        }

        if (isTopic()) {
            e.authorizables = authorizables;
            e.canAssignToMe = canAssignToMe();
            e.canPost = canPost();
        }

		return e;
	}

    def getMetadataAsJson() {
        return JsonOutput.toJson(getMetadata());
    }
}
