import api.cms.Item;
import api.cms.MultipartUpload;
import api.http.WebRequest;
import api.http.WebResponse;
import api.security.User;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def now = new Date();
		def params = WebRequest.create(request).parseRequest();
		if (!params.id) {
			// Bad Request
			response.setStatus(400);
			return;
		}

		def user = User.create(context).findByName(params.id);
		if (!user.exists()) {
			// Not Found
			response.setStatus(404);
			return;
		}

		user.setAttribute("lastModified", now);
		user.setAttribute("lastModifiedBy", repositorySession.userID);
		if (params.photo) {
			if (params.photo.uploadID) {
				def mu = MultipartUpload.create(context).resolve(params.photo.uploadID);
				if (!mu.exists()) {
					// Bad Request
					WebResponse.create(response).setStatus(400);
					return;
				}

				def mimeType = params.photo.mimeType;
				if (!mimeType || mimeType == "application/octet-stream") {
					def type = MimeTypeAPI.getMimeType(mu.filename);
					if (type) {
						mimeType = type;
					}
				}
				if (!mimeType) {
					mimeType = "application/octet-stream";
				}

				user.setAttribute("photo", mu.file.bytes, mimeType);
			}

			if (params.photo.path) {
				def srcItem = Item.create(context).findByPath(params.photo.path);
				if (!srcItem.exists()) {
					// Bad Request
					WebResponse.create(response).setStatus(400);
					return;
				}

				user.setAttribute("photo", srcItem.contentAsByteArray, srcItem.contentType);
			}
		}
		if (params.properties) {
			for (def prop in params.properties) {
				def key = prop.key;
				if (user.contains(key)) {
					user.removeAttribute(key);
				}
				if (prop.value == null) {
					continue;
				}

				def type = prop.type;
				def value = prop.value;
				def mask = !!prop.mask;
				if (type.equalsIgnoreCase("String")) {
					if (value instanceof Collection) {
						user.setAttribute(key, value as String[], mask);
					} else {
						user.setAttribute(key, value as String, mask);
					}
				} else if (type.equalsIgnoreCase("Binary")) {
					if (value instanceof Collection) {
						// Bad Request
						response.setStatus(400);
						return;
					} else {
						user.setAttribute(key, value.decodeBase64());
					}
				} else if (type.equalsIgnoreCase("Long")) {
					if (value instanceof Collection) {
						user.setAttribute(key, value as Long[]);
					} else {
						user.setAttribute(key, value as Long);
					}
				} else if (type.equalsIgnoreCase("Double")) {
					if (value instanceof Collection) {
						user.setAttribute(key, value as Double[]);
					} else {
						user.setAttribute(key, value as Double);
					}
				} else if (type.equalsIgnoreCase("Decimal")) {
					if (value instanceof Collection) {
						user.setAttribute(key, value as BigDecimal[]);
					} else {
						user.setAttribute(key, value as BigDecimal);
					}
				} else if (type.equalsIgnoreCase("Date")) {
					if (value instanceof Collection) {
						def values = [];
						for (def v in value) {
							values.add(ISO8601.parse(v));
						}
						user.setAttribute(key, values as Date[]);
					} else {
						user.setAttribute(key, ISO8601.parse(value));
					}
				} else if (type.equalsIgnoreCase("Boolean")) {
					if (value instanceof Collection) {
						user.setAttribute(key, value as Boolean[]);
					} else {
						user.setAttribute(key, value as Boolean);
					}
				}
			}
		}

		// commit
		repositorySession.commit();

		// OK
		WebResponse.create(response)
			.setStatus(200)
			.setContentType("application/json");
		out.print(user.toJson());
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		// Unauthorized
		response.sendError(401, ex.message);
	}
}();
