import api.http.WebRequest;
import api.http.WebResponse;
import api.util.YAML;
import groovy.json.JsonOutput;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def rootPath = resource.parent.parent.parent.path;
		def appsPath = rootPath + "/apps";
		def stmt = "/jcr:root" + XPath.encodeXML(appsPath) + "//element(app.yml,nt:file)";
		def result = XPath.createQuery(stmt).offset(0).limit(1000).execute();
		def o = [
			"total": result.total,
			"hasMore": result.hasMore(),
			"apps": []
		];
		for (r in result.resources) {
			try {
				def cat = r.parent.parent.getResource("category.yml");
				if (!cat.exists()) {
					continue;
				}

				def app = YAML.parse(r);
				if (app.isAdminOnly) {
					if (!repositorySession.isAdmin()) {
						continue;
					}
				}

				app.name = r.parent.name;
				app.appHome = r.parent.path;
				app.relPath = r.parent.path.substring((appsPath + "/").length());
				app.icon = null;
				if (r.parent.getResource("icon.svg").exists()) {
					app.icon = "icon.svg";
				}
				if (app.actions) {
					app.actions.each { k, v ->
						v.identifier = k;
					}
				}
				app.category = YAML.parse(cat);
				app.category.identifier = cat.parent.path.substring(appsPath.length());
				app.modified = new Date().time;
				def indexFile = context.resourceResolver.getResource(r.parent.path + "/index.html");
				if (indexFile.exists()) {
					app.modified = indexFile.lastModified.time;
				}
				def remainderFile = context.resourceResolver.getResource(r.parent.path + "/remainder.js");
				app.hasRemainderScript = remainderFile.exists();
				o.apps.add(app);
			} catch (Throwable ex) {
				log.warn("Could not parse app.yaml", ex);
			}
		}
		// OK
		WebResponse.create(response)
			.setStatus(200)
			.setContentType("application/json");
		out.print(JsonOutput.toJson(o));
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		WebResponse.create(response).sendError(ex);
	}
}();
