import api.cms.Item;
import api.http.WebRequest;
import api.http.WebResponse;
import api.util.YAML;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def appConfig = YAML.parse(resource.parent.parent.parent.getResource("app.yml"));
		def params = WebRequest.create(request).parseRequest();
		def identifier = params.identifier;
		if (!identifier) {
			// Bad Request
			WebResponse.create(response).setStatus(400);
			return;
		}

		def path = "/var/webtop/home/" + repositorySession.userID + "/appData/" + appConfig.identifier;
		def appFolder = Item.create(context).findByPath(path);
		def item = appFolder.getItem(identifier + ".txt");
		if (!item.exists()) {
			// Not Found
			WebResponse.create(response).setStatus(404);
			return;
		}

		item.remove();

		def imagesFolder = appFolder.getItem(identifier);
		if (imagesFolder.exists()) {
			imagesFolder.remove();
		}

		repositorySession.commit();

		// OK
		WebResponse.create(response)
			.setStatus(204);
		return;
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		response.sendError(500, ex.message);
	} finally {
		try {
			repositorySession.rollback();
		} catch (Throwable ignore) {}
	}
}();
