import api.cms.Item;
import api.cms.ItemHelper;
import api.cms.MultipartUpload;
import api.http.WebRequest;
import api.http.WebResponse;
import api.util.ISO8601;
import api.util.YAML;
import app.memo.Memo;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def appConfig = YAML.parse(resource.parent.parent.parent.getResource("app.yml"));
		def now = new Date();
		def params = WebRequest.create(request).parseRequest();
		def identifier = params.identifier;
		if (!identifier) {
			// Bad Request
			WebResponse.create(response).setStatus(400);
			return;
		}

		def path = "/var/webtop/home/" + repositorySession.userID + "/appData/" + appConfig.identifier;
		def appFolder = Item.create(context).findByPath(path);
		def item = appFolder.getItem(identifier + ".txt");
		if (!item.exists()) {
			// Not Found
			WebResponse.create(response).setStatus(404);
			return;
		}

		if (params.text != null) {
			item.setContent(params.text);
		}
		if (params.tag != null) {
			item.setAttribute("memo.tag", params.tag as String[]);
		}
		if (params.color != null) {
			item.setAttribute("memo.color", params.color);
		}
		if (params.remainder != null) {
			item.setAttribute("memo.remainder.enabled", params.remainder.enabled);
			if (params.remainder.date != null) {
				item.setAttribute("memo.remainder.date", ISO8601.parseDate(params.remainder.date));
			}
		}
		if (params.image != null) {
			def imagesFolder = appFolder.getItem(identifier);
			imagesFolder.mkdirs();
			for (u in params.image) {
				if (u.removed) {
					def imageFile = imagesFolder.getItem(u.filename);
					if (imageFile.exists()) {
						imageFile.remove();
					}
					continue;
				}

				if (u.uploadID) {
					def mu = MultipartUpload.create(context).resolve(u.uploadID);
					if (!mu.exists()) {
						// Bad Request
						WebResponse.create(response).setStatus(400);
						return;
					}

					def imageFile = imagesFolder.getItem(u.filename);
					if (!imageFile.exists()) {
						imageFile.createNewFile();
					}
					ItemHelper.create(context).with(imageFile).importContent(mu, u.mimeType);
					imageFile.setAttribute("jcr:lastModified", now);
					continue;
				}

				if (u.path) {
					def srcItem = Item.create(context).findByPath(u.path);
					if (!srcItem.exists()) {
						// Bad Request
						WebResponse.create(response).setStatus(400);
						return;
					}

					def imageFile = imagesFolder.getItem(srcItem.name);
					if (imageFile.path == srcItem.path) {
						// Bad Request
						WebResponse.create(response).setStatus(400);
						return;
					}
					if (!imageFile.exists()) {
						imageFile.createNewFile();
					}
					srcItem.copyTo(imageFile.path);
					imageFile.setAttribute("jcr:lastModified", now);
					continue;
				}
			}
		}

		item.setAttribute("jcr:lastModified", now);
		repositorySession.commit();

		def memoItem = Memo.create(context).with(item.resource);

		// OK
		WebResponse.create(response)
			.setStatus(200)
			.setContentType("application/json");
		out.print(memoItem.metadataAsJson);
		return;
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		response.sendError(500, ex.message);
	} finally {
		try {
			repositorySession.rollback();
		} catch (Throwable ignore) {}
	}
}();
