import api.cms.Item;
import api.http.WebRequest;
import api.http.WebResponse;
import api.util.YAML;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def appConfig = YAML.parse(resource.parent.parent.parent.getResource("app.yml"));
		def identifier = WebAPI.getParameter("identifier").defaultString().trim();
		def filename = WebAPI.getParameter("filename").defaultString().trim();
		def range = WebRequest.create(request).range;
		if (!identifier || !filename || (range && !range.isValid())) {
			// Bad Request
			WebResponse.create(response).setStatus(400);
			return;
		}

		def path = "/var/webtop/home/" + repositorySession.userID + "/appData/" + appConfig.identifier;
		def appFolder = Item.create(context).findByPath(path);
		def imagesFolder = appFolder.getItem(identifier);
		def imageFile = imagesFolder.getItem(filename);
		if (!imageFile.exists()) {
			// Not Found
			WebResponse.create(response).setStatus(404);
			return;
		}

		WebResponse
			.create(response)
			.setStatus(200)
			.setAttachment(imageFile.name)
			.disableContentCache()
			.setContentType(imageFile.contentType)
			.setCharacterEncoding(imageFile.contentEncoding)
			.setContentLength(imageFile.contentLength)
			.setETag(imageFile.lastModified.time as String)
			.writePartial(imageFile.contentAsStream, range);
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		response.sendError(500, ex.message);
	}
}();
