import api.http.WebRequest;
import api.http.WebResponse;
import api.util.ISO8601;
import groovy.json.JsonOutput;
import org.apache.commons.csv.CSVFormat;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def params = WebRequest.create(request).parseRequest();
		if (!params.path) {
			// Bad Request
			response.setStatus(400);
			return;
		}

		def cnd = '';
		if (params.keyword) {
			cnd += "jcr:contains(jcr:content," + XPath.value(XPath.term(params.keyword)) + ")";
		}
		if (params.variables != null) {
			params.variables.each { e ->
				def variable = e.value;
				def xmlname = "jcr:content/@" + XPath.encodeXMLComponent(variable.key);
				if (variable.valueType == "have") {
					if (cnd) {
						cnd += " and ";
					}
					cnd += xmlname;
					return;
				}

				if (variable.valueType == "nothave") {
					if (cnd) {
						cnd += " and ";
					}
					cnd += "not(" + xmlname + ")";
					return;
				}

				if (variable.type == "String" && variable.value instanceof List) {
					if (variable.valueType == "containing_all" || variable.valueType == "containing_any") {
						if (cnd) {
							cnd += " and ";
						}
						def op = (variable.valueType == "containing_all") ? " and " : " or ";
						def c = "";
						variable.value.each { value ->
							if (!value) {
								return;
							}

							if (c) {
								c += op;
							}
							c += xmlname + " = " + XPath.value(value.trim());
						}
						c = "(" + c + ")";
						if (variable.isNot) {
							c = "not" + c;
						}
						cnd += c;
						return;
					}

					return;
				}

				if (variable.type == "String") {
					if (variable.valueType == "containing_text") {
						if (cnd) {
							cnd += " and ";
						}
						def c = "jcr:contains(" + xmlname + "," + XPath.value(XPath.term(variable.value)) + ")";
						if (variable.isNot) {
							c = "not(" + c + ")";
						}
						cnd += c;
						return;
					}

					if (variable.valueType == "containing_all" || variable.valueType == "containing_any") {
						if (cnd) {
							cnd += " and ";
						}
						def op = (variable.valueType == "containing_all") ? " and " : " or ";
						def csv = CSVFormat.DEFAULT
							.withDelimiter(' ' as char)
							.withIgnoreEmptyLines(true)
							.withTrim(true)
							.parse(new StringReader(variable.value));
						def c = "";
						csv.each { record ->
							record.each { column ->
								if (!column) {
									return;
								}

								if (c) {
									c += op;
								}
								if (column.startsWith('%') || column.endsWith('%')) {
									c += "jcr:like(" + xmlname + "," + XPath.value(column.trim()) + ")";
								} else if (column.startsWith('{{') || column.endsWith('}}')) {
									column = column.substring(2, column.length - 2);
									c += "jcr:contains(" + xmlname + "," + XPath.value(XPath.term(column.trim())) + ")";
								} else {
									c += xmlname + " = " + XPath.value(column.trim());
								}
							}
						}
						c = "(" + c + ")";
						if (variable.isNot) {
							c = "not" + c;
						}
						cnd += c;
						return;
					}

					return;
				}

				if (variable.type == "Decimal") {
					if (cnd) {
						cnd += " and ";
					}

					def from = "";
					if (variable.valueFrom != null) {
						def op = variable.greaterThan ? ">" : ">=";
						from = xmlname + " " + op + " " + XPath.value(variable.valueFrom as BigDecimal);
					}

					def to = "";
					if (variable.valueTo != null) {
						def op = variable.lessThan ? "<" : "<=";
						to = xmlname + " " + op + " " + XPath.value(variable.valueTo as BigDecimal);
					}

					def c = from;
					if (to) {
						if (c) {
							c += " and ";
						}
						c += to;
					}
					c = "(" + c + ")";
					if (variable.isNot) {
						c = "not" + c;
					}
					cnd += c;
					return;
				}

				if (variable.type == "Date") {
					if (cnd) {
						cnd += " and ";
					}

					def from = "";
					if (variable.valueFrom != null) {
						def op = variable.greaterThan ? ">" : ">=";
						from = xmlname + " " + op + " " + XPath.value(ISO8601.parseDate(variable.valueFrom));
					}

					def to = "";
					if (variable.valueTo != null) {
						def op = variable.lessThan ? "<" : "<=";
						to = xmlname + " " + op + " " + XPath.value(ISO8601.parseDate(variable.valueTo));
					}

					def c = from;
					if (to) {
						if (c) {
							c += " and ";
						}
						c += to;
					}
					c = "(" + c + ")";
					if (variable.isNot) {
						c = "not" + c;
					}
					cnd += c;
					return;
				}

				if (variable.type == "Boolean") {
					if (cnd) {
						cnd += " and ";
					}
					def c = xmlname + " = " + XPath.value(variable.value as boolean);
					cnd += c;
					return;
				}
			}
		}

		def stmt = "/jcr:root";
		if (params.path && params.path != '/') {
			stmt += XPath.encodeXML(params.path);
		}
		stmt += "//element(*,nt:file)";
		if (cnd) {
			stmt += "[" + cnd + "]";
		}
		stmt += " order by @jcr:path";

		def resp = [
			"statement": stmt
		];

		// OK
		WebResponse.create(response)
			.setStatus(200)
			.setContentType("application/json");
		out.print(JsonOutput.toJson(resp));
		return;
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		response.sendError(500, ex.message);
	} finally {
		try {
			repositorySession.rollback();
		} catch (Throwable ignore) {}
	}
}();
