import api.cms.Item;
import api.http.WebRequest;
import api.http.WebResponse;
import api.util.ISO8601;
import api.util.YAML;
import groovy.json.JsonOutput;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def appConfig = YAML.parse(resource.parent.parent.parent.getResource("app.yml"));
		def params = WebRequest.create(request).parseRequest();
		def resp = [
			"memos": []
		];

		def stmt = "/jcr:root/var/webtop/home/" + XPath.encodeXMLComponent(repositorySession.userID) + "/appData/" + XPath.encodeXMLComponent(appConfig.identifier) + "/element(*, nt:file)[";
		stmt += "jcr:content/@memo.identifier";
		stmt += " and jcr:content/@memo.remainder.enabled = " + XPath.value(true);
		stmt += " and jcr:content/@memo.remainder.date";
		if (params.dateAfter) {
			stmt += " and jcr:content/@memo.remainder.date >= " + XPath.value(ISO8601.parseDate(params.dateAfter));
		}
		if (params.dateBefore) {
			stmt += " and jcr:content/@memo.remainder.date <= " + XPath.value(ISO8601.parseDate(params.dateBefore));
		}
		stmt += "]";
		stmt += " order by jcr:content/@memo.remainder.date";
		XPath.createQuery(stmt);
		def result = XPath.createQuery(stmt).execute();
		for (r in result.resources) {
			def item = Item.create(context).with(r);
			def identifier = item.name.substring(0, item.name.length() - 4);
			def e = [
				"identifier": identifier,
				"text": item.content,
				"color": item.getString("memo.color"),
				"tag": item.getStringArray("memo.tag"),
				"image": [],
				"created": ISO8601.formatDate(item.created),
				"lastModified": ISO8601.formatDate(item.lastModified)
			];
			def imagesFolder = item.parent.getItem(identifier);
			if (imagesFolder.exists() && imagesFolder.isCollection()) {
				for (imageFile in imagesFolder.list()) {
					if (imageFile.isCollection()) {
						continue;
					}
					e.image.add([
						"filename": imageFile.name,
						"mimeType": imageFile.contentType,
						"size": imageFile.contentLength,
						"path": imageFile.path,
						"lastModified": ISO8601.formatDate(imageFile.lastModified),
						"item": imageFile.toObject()
					]);
				}
			}
			if (item.contains("memo.remainder.enabled")) {
				e.remainder = [
					"enabled": item.getBoolean("memo.remainder.enabled")
				];
				if (item.contains("memo.remainder.date")) {
					e.remainder["date"] = ISO8601.formatDate(item.getDate("memo.remainder.date"));
				}
			}
			resp.memos.add(e);
		}

		// OK
		WebResponse.create(response)
			.setStatus(200)
			.setContentType("application/json");
		out.print(JsonOutput.toJson(resp));
		return;
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		response.sendError(500, ex.message);
	}
}();
