export default class Strings {
	static quillSubject(v, defaultValue) {
		if (defaultValue == undefined) {
			defaultValue = '';
		}

		if (v == undefined) {
			return defaultValue;
		}

		try {
			v = JSON.parse(v);
		} catch (ex) {
			// ignore
		}

		if (typeof v == 'object') {
			let headers = [];
			for (let i = 0; i < v.ops.length; i++) {
				let e = v.ops[i];
				if (e.attributes && e.attributes.header != undefined) {
					headers.push({
						'index': i,
						'header': e.attributes.header
					});
				}
			}
			if (headers.length > 0) {
				headers.sort(function(a, b) {
					if (a.header < b.header) {
						return -1;
					}
					if (a.header > b.header) {
						return 1;
					}
					return 0;
				});
				let text = '';
				for (let i = headers[0].index - 1; i >= 0; i--) {
					let e = v.ops[i];
					if (e.insert != undefined) {
						text = e.insert;
						break;
					}
				}
				let p = text.lastIndexOf('\n');
				if (p != -1) {
					text = text.substring(p + 1);
				}
				return text;
			}
		}

		let text = '';
		if (typeof v == 'object') {
			for (let e of v.ops) {
				if (e.insert != undefined) {
					text += e.insert;
				}
			}
		} else {
			text = v;
		}
		let lines = text
			.replace(/\r\n/g, "\n")
			.replace(/\r/g, "\n")
			.split('\n');
		for (let line of lines) {
			let t = line.trim();
			if (t) {
				return t;
			}
		}
		return defaultValue;
	}

	static quillContent(v, defaultValue) {
		if (defaultValue == undefined) {
			defaultValue = '';
		}

		if (v == undefined) {
			return defaultValue;
		}

		try {
			v = JSON.parse(v);
		} catch (ex) {
			// ignore
		}

		if (typeof v == 'object') {
			let headers = [];
			for (let i = 0; i < v.ops.length; i++) {
				let e = v.ops[i];
				if (e.attributes && e.attributes.header != undefined) {
					headers.push({
						'index': i,
						'header': e.attributes.header
					});
				}
			}
			if (headers.length > 0) {
				headers.sort(function(a, b) {
					if (a.header < b.header) {
						return -1;
					}
					if (a.header > b.header) {
						return 1;
					}
					return 0;
				});
				let text = '';
				for (let i = headers[0].index; i < v.ops.length; i++) {
					let e = v.ops[i];
					if (e.insert != undefined) {
						text += e.insert;
					}
				}
				let lines = text
					.replace(/\r\n/g, "\n")
					.replace(/\r/g, "\n")
					.split('\n');
				let message = '';
				for (let line of lines) {
					let t = line.trim();
					if (t) {
						if (message) {
							message += ' ';
						}
						message += t;
					}
				}
				return message;
			}
		}

		let text = '';
		if (typeof v == 'object') {
			for (let e of v.ops) {
				if (e.insert != undefined) {
					text += e.insert;
				}
			}
		} else {
			text = v;
		}
		let lines = text
			.replace(/\r\n/g, "\n")
			.replace(/\r/g, "\n")
			.split('\n');
		let message = '';
		let subjectSkipped = false;
		for (let line of lines) {
			let t = line.trim();
			if (t) {
				if (!subjectSkipped) {
					subjectSkipped = true;
					continue;
				}
				if (message) {
					message += ' ';
				}
				message += t;
			}
		}
		return message;
	}
}

export class Observer {
	constructor(context) {
		this.$context = context;
	}

	get appConfig() {
		return this.$context.appConfig;
	}

	get baseURL() {
		let url = import.meta.url;
		return url.substring(0, url.lastIndexOf('/'));
	}

	observe(options) {
		let instance = this;
		return instance.$context.axios.post(instance.baseURL + '/js/Observer/Observe.groovy', options).then(function(response) {
			let l = [];
			for (let memo of response.data.memos) {
				let e = {
					'identifier': memo.identifier,
					'title': Strings.quillSubject(memo.text),
					'memo': Strings.quillContent(memo.text),
					'date': new Date(memo.remainder.date),
					'applicationIdentifier': instance.appConfig.identifier,
					'launchOptions': {
						'identifier': memo.identifier
					},
					'canLaunch': true
				};
				l.push(e);
			}
			return l;
		});
	}
}
