import api.cms.Item;
import api.http.WebRequest;
import api.http.WebResponse;
import api.util.YAML;
import app.task.Task;

{->
	if (repositorySession.isAnonymous()) {
		// Unauthorized
		response.setStatus(401);
		return;
	}

	try {
		def appConfig = YAML.parse(resource.parent.parent.parent.getResource("app.yml"));
		def now = new Date();
		def params = WebRequest.create(request).parseRequest();
		if (!params.identifier) {
			// Bad Request
			response.setStatus(400);
			return;
		}

		def task = Task.create(context, appConfig).findByIdentifier(params.identifier);
		if (!task.exists()) {
			// Not Found
			response.setStatus(404);
			return;
		}

		if (params.containsKey('subject')) {
			task.setSubject(params.subject);
		}
		if (params.containsKey('comments')) {
			task.setComments(params.comments);
		}
		if (params.containsKey('color')) {
			task.setColor(params.color);
		}
		if (params.containsKey('tag')) {
			task.setTags(params.tag);
		}
		if (params.containsKey('status')) {
			task.setStatus(params.status);
		}
		if (params.containsKey('dueDate')) {
			task.setDueDate(params.dueDate);
		}
		if (params.containsKey('followUpDate')) {
			task.setFollowUpDate(params.followUpDate);
		}
		if (params.containsKey('assignee')) {
			task.setAssignee(params.assignee);
		}
		if (params.containsKey('candidateUser')) {
			task.setCandidateUsers(params.candidateUser);
		}
		if (params.containsKey('candidateGroup')) {
			task.setCandidateGroups(params.candidateGroup);
		}
		if (params.containsKey('isDraft')) {
			task.setDraft(params.isDraft);
		}
		if (params.containsKey('attachments')) {
			task.setAttachments(params.attachments);
		}

		task.setAttribute("jcr:lastModified", now);
		repositorySession.commit();

		// OK
		WebResponse.create(response)
			.setStatus(200)
			.setContentType("application/json");
		out.print(task.metadataAsJson);
		return;
	} catch (Throwable ex) {
		log.error(ex.message, ex);
		response.sendError(500, ex.message);
	} finally {
		try {
			repositorySession.rollback();
		} catch (Throwable ignore) {}
	}
}();
